// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "OAuthToken.h"

@implementation OAuthToken

+(id)oauthTokenWith:(NSString *)accessToken expirationDate:(NSDate*)date
       refreshToken:(NSString *)refreshToken{
    
    OAuthToken* token= [[OAuthToken alloc] init];
    token.accessToken= accessToken;
    token.type= @"bearer";
    token.refreshToken= refreshToken;
    token.expirationDate= date;
    
    return token;
}

-(NSString *)description{

    NSMutableString* desc= [[NSMutableString alloc] init];
    
    [desc appendFormat:@"access token : %@\nrefresh token: %@\nexpires      : %@\ntype         : %@\n", _accessToken,_refreshToken, _expirationDate, _type];
    
    return desc;
}

-(BOOL)isExpired{
    
    // Compare today's date with the expiration date
    NSDate* today= [NSDate date];
    NSDate* earlierDate= [_expirationDate earlierDate:today];
    // if the expiration date is older/earlier then today's date
    if (earlierDate==_expirationDate) {
        // the token has expired
        return YES;
    }
    
    // the token is still valid.
    return NO;
}

@end
