// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "ATTRequest.h"
#import <AFHTTPRequestOperation.h>

NSString* const IAMTrue= @"true";
NSString* const IAMFalse= @"false";
NSString* const RTJson= @"JSON";

@implementation ATTRequest

@synthesize method= _method;
@synthesize path= _path;

- (id)init
{
    self = [super init];
    if (self) {
        // set default value for properties
        _method= @"GET";
        _path= @"";
        _responseType= @"JSON";
        
    }
    return self;
}

-(NSDictionary *)params{
    return [self prepareParameters];
}

#pragma mark - Prepare parameters dictionary

-(NSMutableDictionary *)prepareParameters{
    // for this class only return an emtpy dictionary
    return [NSMutableDictionary dictionary];
}

/** Base implementation for parsing responses. 
  If the code reaches this method it means that the method has not been 
  implemented in the sub-type that made the call and it should report this 
  as an error. */
-(id)parseResponse:(id)response forOperation:(AFHTTPRequestOperation*)operation
             error:(NSError *__autoreleasing *)error{
    
    NSString* errorMsg= [NSString stringWithFormat:
                         @"No parsing rules exist for request of type %@."
                         @"\nPlease implement parsing rules for this response type.",
                         NSStringFromClass([self class])];
    
    if (error) {
        *error= [NSError errorWithDomain:@"NoParsingRuleDomain"
                                    code:101
                                userInfo:@{NSLocalizedDescriptionKey: errorMsg}];
    }
    
    return nil;
}
@end
