// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "IAMDeleteMessagesRequest.h"
#import "IAMMessage.h"
#import <AFHTTPRequestOperation.h>

@implementation IAMDeleteMessagesRequest

@synthesize method=_method;
@synthesize path=_path;

- (id)init
{
    self = [super init];
    if (!self) {
        return nil;
    }
    
    self= [self initWithMessages:nil];
    
    return self;
}

- (id)initWithMessages:(NSArray *)messages{
    
    self = [super init];
    if (!self) {
        return nil;
    }
    
    _method= @"DELETE";
    _path= @"/myMessages/v2/messages";
    _messages= messages;
    
    return self;
}

-(NSMutableDictionary *)prepareParameters{
    NSMutableDictionary* requestParams= [super prepareParameters];
    
    (requestParams)[@"messageIds"]= [self idsStringFromMessages:_messages];
    
    return requestParams;
}

-(NSString*)idsStringFromMessages:(NSArray*)messages{
    
    NSMutableArray* idsArray= [NSMutableArray array];
    
    for (IAMMessage* iamMessage in messages) {
        [idsArray addObject:iamMessage.messageId];
    }

    return [idsArray componentsJoinedByString:@","];
}

-(id)parseResponse:(id)response forOperation:(AFHTTPRequestOperation *)operation error:(NSError **)error{

    // We expect a 204 status code on successful deletion.
    if (204 != operation.response.statusCode) {
        if (error) {
            NSString* description= [NSString stringWithFormat:
                                    @"Deletion failed."];
            *error= [NSError errorWithDomain:@"ParsingDomain"
                                        code:101
                                    userInfo:@{NSLocalizedDescriptionKey: description}];
            
            return [NSNumber numberWithBool:NO];
        }
    }
    
    // the deletion was successful
    return [NSNumber numberWithBool:YES];

    
}
@end
