// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import <AFNetworking/AFNetworking.h>


#import "IAMManager.h"

#import "ATTRequest.h"
#import "IAMMessagesListRequest.h"
#import "IAMMessageIndexInfoRequest.h"
#import "IAMDeltasUpdateRequest.h"
#import "IAMSendMessageRequest.h"

#import "IAMMessageIndexInfo.h"
#import "IAMDeltasUpdateResponse.h"

#import "JSONParser.h"

@interface IAMManager ()

@end

@implementation IAMManager

@synthesize mailboxState= _mailboxState;

#pragma mark - Convience methos for making common requests

-(void)sendSMS:(NSString*)text to:(NSString*)addresses
       failure:(void (^)(NSError*))failure{
    
    IAMSendMessageRequest* sendSMSRequest= [[IAMSendMessageRequest alloc] init];
    sendSMSRequest.addresses= [addresses componentsSeparatedByString:@","];
    sendSMSRequest.text= text;
    
    [self sendAsynchronous:sendSMSRequest success:^(id messageId) {
        // nothing
        NSLog(@"Message succesfully sent with Id: %@.", messageId);
    } failure:^(NSError *error) {
        failure(error);
    }];
}

-(void)getAllMessages:(void (^)(NSArray *))success failure:(void (^)(NSError *))failure{

    IAMMessagesListRequest* request= [[IAMMessagesListRequest alloc] init];
    
    // Call the more general method to execute the actual request
    [self sendAsynchronous:request success:^(id messages) {
        success(messages);
    } failure:^(NSError *error) {
        failure(error);
    }];
}


#pragma Custom IAM-specific behaviour

-(void)updateMessageIndexInfo:(IAMMessageIndexInfo*)messageIndexInfo{
    _mailboxState = messageIndexInfo.mailboxState;
}

/// Allows the interception of the response, to enable customization on
/// the parsed object handed to the developer.
/// TODO: NOTE: To disable this custom behavior and get the parsed responses
/// AS-IS, just comment this overriden method.
-(id)parsedObjectForSuccessfulOperation:(AFHTTPRequestOperation *)operation
                           withResponse:(id)response
                          forATTRequest:(ATTRequest *)attRequest
                handleParsingErrorsWith:(void (^)(NSError *))failure{

    id parsedResponse= [super parsedObjectForSuccessfulOperation:operation
                                                    withResponse:response
                                                   forATTRequest:attRequest
                                         handleParsingErrorsWith:failure];
    
    // here you can limit/customize what is the final
    // parsed object that will reach the developer.
    
    // For example you may want to hide some ATTRequestManager state changes,
    // like the inbox state.
    
    // if this is requesting the messageIndexInfo update the IAMManager state
    // immediately with the resulting data
    if ([attRequest isKindOfClass:[IAMMessageIndexInfoRequest class]]) {
        // update the state of the IAMManager
        [self updateMessageIndexInfo:parsedResponse];
    }
    
    // you may want to update the mailbox state and just hand the updates
    // to the developer.
    if ([attRequest isKindOfClass:[IAMDeltasUpdateRequest class]]) {
        
        IAMDeltasUpdateResponse* mailboxDeltas= parsedResponse;
        // update the mailbox state
        [self updateMessageIndexInfo:mailboxDeltas.messageIndexInfo];
        
        // hand to the developer only the arra of deltas
        return mailboxDeltas.deltas;
    }
    
    // This is just a just pass-through for the rest of the request types
    return parsedResponse;
}

@end
