// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import <UIKit/UIKit.h>
#import "IAMMessageContentRequest.h"
#import <AFHTTPRequestOperation.h>

@implementation IAMMessageContentRequest

@synthesize path=_path;
@synthesize method=_method;
@synthesize responseType=_responseType;

-(id)init{
    self = [super init];
    
    if (!self) {
        return nil;
    }
    
    _path= @"myMessages/v2/messages/";
    _method= @"GET";
    _partNumber= 0;
    _responseType= @"DATA";
    
    return self;
}

-(NSString *)path{
    NSString* relativePath= [NSString stringWithFormat:@"%@/parts/%ld",
                             _messageId, (long)_partNumber];
    
    return [_path stringByAppendingString:relativePath];
}

-(id)parseResponse:(id)response forOperation:(AFHTTPRequestOperation *)operation
             error:(NSError **)error{

    // if the reponse is an image, return the native UIImage object
    NSRange imageRange= [operation.response.MIMEType rangeOfString:@"image"
                                                           options:NSCaseInsensitiveSearch];
    if (NSNotFound != imageRange.location) {
        UIImage* image= [UIImage imageWithData: response];
        
        // report errors reading the image binary data
        if (!image) {
            if (error) {
                NSString* description= @"Could not create Image from response.";
                *error= [NSError errorWithDomain:@"ParsingDomain"
                                                         code:102
                                                     userInfo:@{NSLocalizedDescriptionKey: description}];
                return nil;
            }
        }
        // if there were no errors return the UIImage object
        return image;
    }
    
    // If the response is not an image,
    // assume it's plain text and return it's string representation
    NSString* responseString= [[NSString alloc] initWithData:response encoding:NSUTF8StringEncoding];
    if (!responseString) {
        if (error) {
            NSString* description= @"Could not create NSString from response.";
            *error= [NSError errorWithDomain:@"ParsingDomain"
                                        code:102
                                    userInfo:@{NSLocalizedDescriptionKey: description}];
            return nil;
        }
    }
    
    return  responseString;
}

@end
