// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "IAMMessageRequest.h"
#import "JSONParser.h"
#import "IAMMessage.h"

@implementation IAMMessageRequest

@synthesize path=_path;

-(id)init{
    
    self= [super init];
    
    if(!self) return nil;
    
    // initialize resource specific params
    _path= @"/myMessages/v2/messages/";
    _messageID= @"";
    
    return self;
}

-(NSString *)path{
    return [_path stringByAppendingString: _messageID];
}

-(id)parseResponse:(id)JSONResponse forOperation:(AFHTTPRequestOperation *)operation
             error:(NSError **)error{

    // get the message element from the response
    NSDictionary* messageJSON= (JSONResponse)[@"message"];
    
    // report an error if there's no message element
    if (!messageJSON) {
        if (error) {
            NSString* description= [NSString stringWithFormat:
                                    @"Error parsing message: "
                                    @"No `message` element in response: %@", JSONResponse];
            
            *error= [NSError errorWithDomain:@"ParsingDomain"
                                        code:100
                                    userInfo:@{NSLocalizedDescriptionKey: description}];
        }
        return nil;
    }
    
    IAMMessage* iamMessage= [JSONParser getMessageFromJSON:messageJSON];
    
    // if the message could not be parsed
    if (!iamMessage) {
        
        // fill in the error details
        
        if (error) {
            NSString* description= [NSString stringWithFormat:
                                    @"Error parsing message: %@", JSONResponse];
            
            *error= [NSError errorWithDomain:@"ParsingDomain"
                                       code:100
                                   userInfo:@{NSLocalizedDescriptionKey: description}];
        }
        // return early with nil value
        return nil;
    }
    
    // by this point we should have a valid IAMMessage object
    return iamMessage;
}
@end
