// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

NSString* const IAMResultsLimit= @"limit";
NSString* const IAMResultsOffset= @"offset";

NSString* const IAMMessageIDsFilter= @"messageIds";
NSString* const IAMIsUnreadFilter= @"isUnread";
NSString* const IAMMessageTypeFilter= @"type";
NSString* const IAMKeywordFilter= @"keyword";
NSString* const IAMIsIncomingFilter= @"isIncoming";

#import "IAMMessagesListRequest.h"
#import "JSONParser.h"
#import "IAMUtils.h"

@implementation IAMMessagesListRequest


/** Documentation 
 */
- (id)init
{
    self = [super init];
    if (self) {
        
        self.method= @"GET";
        self.path= @"/myMessages/v2/messages";
        
        // set default values
        _limit= 500;
        _offset= 0;
        
    }
    return self;
}

-(NSMutableDictionary*)prepareParameters{
    
    
    NSMutableDictionary* requestParams= nil;

    // take whatever parameters in the base class
    requestParams= [super prepareParameters];
    
    // add the parameters specific for this request
    // don't exceed the limit of 500 messages
    (requestParams)[IAMResultsLimit]= [NSNumber numberWithInteger:
                                       (500<_limit? 500: _limit)];
    (requestParams)[IAMResultsOffset]= [NSNumber numberWithInteger:_offset];
    
    
    // optional parameters, only add them if they are defined
    if (_messageIDsFilter) {
        (requestParams)[IAMMessageIDsFilter]= [_messageIDsFilter componentsJoinedByString:@","];
    }
    if (_filterUnread) {
        (requestParams)[IAMIsUnreadFilter]= IAMTrue;
    }
    if (_typesFilter) {
        (requestParams)[IAMMessageTypeFilter]= [IAMUtils stringFromTypesFilter:_typesFilter];
    }
    if (_keywordFilter) {
        (requestParams)[IAMKeywordFilter]= _keywordFilter;
    }
    if (_incoming) {
        (requestParams)[IAMIsIncomingFilter]= IAMTrue;
    }
    return requestParams;
}

-(id)parseResponse:(id)JSONResponse forOperation:(AFHTTPRequestOperation *)operation
             error:(NSError **)error{

    NSArray* messages= [JSONParser getMessageListFromJSON:JSONResponse];

    // if there are no messages return emtpy array
    if (0 == [messages count]) {
        // report the error
        
        if(error){// check that error has been defined somewhere
            
            // fill the error details
            NSString* errorDescription = @"No messages could be parsed. Emtpy response.";
            *error= [NSError errorWithDomain:@"EmptyResultDomain"
                                        code:100
                                    userInfo:@{NSLocalizedDescriptionKey: errorDescription}];
            
            // return emtpy array
            return @[];
        }

    }
    
    // by this point, we should have a non-empty array of IAMMessage objects
    return messages;
}

@end
