// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "IAMSendMessageRequest.h"
#import <AFHTTPRequestOperation.h>

#import <UIKit/UIKit.h>

#if defined(__IPHONE_6_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_6_1
#import "Base64.h"
#else
#error Your SDK is too old ! Need at least 6.0
#endif

NSString* const IAMImageAttachment= @"image/png";
NSString* const IAMTextAttachment= @"plain/text";

@implementation IAMSendMessageRequest
{
    NSMutableArray* attachments;
}

- (id)init
{
    self= [super init];
    
    if(!self){
        return nil;
    }
    
    self.method= @"POST";
    self.path= @"/myMessages/v2/messages";
    
    _isGroup= YES;
    
    return self;
}

-(NSMutableDictionary *)prepareParameters{

    NSMutableDictionary* requestParams= [super prepareParameters];
    
    NSMutableDictionary* messageRequest= [NSMutableDictionary dictionary];
    
    if (_addresses) {
        (messageRequest)[@"addresses"]= _addresses;
    }
    
    if (_text) {
        (messageRequest)[@"text"]= _text;
    }
    
    if (_subject) {
        (messageRequest)[@"subject"]= _subject;
    }
    
    (messageRequest)[@"isGroup"]= _isGroup? IAMTrue: IAMFalse;
    
    // only add attachments to the body if there are any
    if (attachments && 0<attachments.count) {
         (messageRequest)[@"messageContent"]= attachments;
    }
    
    (requestParams)[@"messageRequest"]= messageRequest;
    
    return requestParams;
}

-(id)parseResponse:(id)response forOperation:(AFHTTPRequestOperation *)operation
             error:(NSError **)error{
    
    NSString* messageId= (response)[@"id"];
    if (!messageId) {
        if (error) {
            NSString* description = [NSString stringWithFormat:
                                     @"Invalid messageId received: %@",
                                     response];
            
            *error= [NSError errorWithDomain:@"ParsingDomain" code:101
                                    userInfo: @{NSLocalizedDescriptionKey:description}];
            
            return nil;
        }
    }

    return messageId;
}

#pragma mark - Adding Attachments to the request

-(void)addText:(NSString*)text{
    
    if (!attachments) {
        attachments= [NSMutableArray array];
    }
    
    // Convert string to it's base64 representation
    NSString* base64String= [text base64EncodedString];
    
    NSUInteger curAttachment= attachments.count;
    NSString* fileName= [NSString stringWithFormat:@"file%lu.txt", (unsigned long)curAttachment];
    NSDictionary* textAttachment= [self createAttachment:fileName
                                                    type:IAMTextAttachment
                                                    body:base64String];
    [attachments addObject:textAttachment];
}

-(void)addImage:(UIImage *)image{
    
    if (!attachments) {
        attachments= [NSMutableArray array];
    }
    
#if defined(__IPHONE_6_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_6_1
    NSString* base64String= [UIImagePNGRepresentation(image) base64EncodedString];
#elif defined(__IPHONE_7_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_7_0
    NSString* base64String= [UIImagePNGRepresentation(image) base64EncodedStringWithOptions:0];
#endif
    
    NSUInteger curAttachment= attachments.count;
    NSString* fileName= [NSString stringWithFormat:@"file%lu.png", (unsigned long)curAttachment];
    NSDictionary* imageAttachment= [self createAttachment:fileName
                                                     type:IAMImageAttachment
                                    body:base64String];
    
    [attachments addObject:imageAttachment];
}

-(NSDictionary*)createAttachment:(NSString*)fileName
                            type:(NSString*)type
                            body:(NSString*)body{
    
    NSDictionary* attachment= @{@"fileName": fileName,
                                @"content-type": type,
                                @"content-transfer-encoding": @"BASE64",
                                @"body": body};
    
    return attachment;
}

@end
