// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "IAMUpdateMessageRequest.h"
#import <AFHTTPRequestOperation.h>

@implementation IAMUpdateMessageRequest{
    // Flags to indicates weather or not to add these
    // parameters to the params dictionary
    // the default behavior is not to add them
    // unless the user sets values for them.
    BOOL shouldAddIsUnread;
    BOOL shouldAddIsFavorite;
}

@synthesize method=_method;
@synthesize path=_path;
@synthesize messageId=_messageId;

- (id)init
{
    self = [super init];
    
    if (self) {
        _method= @"PUT";
        _path= @"/myMessages/v2/messages/";
        _isUnread= YES;
        _isFavorite=NO;
        shouldAddIsUnread=NO;
        shouldAddIsFavorite=NO;
    }
    
    return self;
}

-(NSString *)path{
    return [_path stringByAppendingString:_messageId];
}

-(void)setIsUnread:(BOOL)isUnread{
    _isUnread= isUnread;
    shouldAddIsUnread= YES;
}

-(void)setIsFavorite:(BOOL)isFavorite{
    _isFavorite= isFavorite;
    shouldAddIsFavorite= YES;
}

-(NSMutableDictionary *)prepareParameters{

    NSMutableDictionary* requestParams= [super prepareParameters];
    
    if (!_messageId) {
        NSLog(@"You must specify a messageId for this request.");
        return nil;
    }
    
    NSMutableDictionary* message= [NSMutableDictionary dictionary];
    
    // Only add the flag parameters if ther developer has
    // assigned values to them
    if (shouldAddIsUnread) {
        (message)[@"isUnread"]= _isUnread? IAMTrue: IAMFalse;
    }
    
    if (shouldAddIsFavorite) {
        (message)[@"isFavorite"]= _isFavorite? IAMTrue: IAMFalse;
    }

    (requestParams)[@"message"]= message;
    
    return requestParams;
};

-(id)parseResponse:(id)response forOperation:(AFHTTPRequestOperation *)operation
             error:(NSError *__autoreleasing *)error{
    
    // We expect a 204 status code on successful deletion.
    if (204 != operation.response.statusCode) {
        if (error) {
            NSString* description= [NSString stringWithFormat:
                                    @"Update failed for message with: %@",
                                    _messageId];
            *error= [NSError errorWithDomain:@"ParsingDomain"
                                        code:101
                                    userInfo:@{NSLocalizedDescriptionKey: description}];
            
            return [NSNumber numberWithBool:NO];
        }
    }
    
    // the deletion was successful
    return [NSNumber numberWithBool:YES];
}

@end
