// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "IAMUpdateMessagesRequest.h"
#import <AFHTTPRequestOperation.h>
#import "IAMMessage.h"

@implementation IAMUpdateMessagesRequest

-(id)init{
    self= [super init];
    
    if (!self) {
        return nil;
    }
    
    self.method= @"PUT";
    self.path=@"/myMessages/v2/messages";
    
    return self;
}

-(id)initWithMessages:(NSArray *)messages{

    self= [super init];
    
    if (!self) {
        return nil;
    }
    _messages= messages;
    
    return self;
}

-(NSMutableDictionary *)prepareParameters{
    NSMutableDictionary* requestParams= [super prepareParameters];
    
    NSMutableArray* messages= [NSMutableArray array];
    
    for (IAMMessage* iamMessage in _messages) {
        [messages addObject:@{@"messageId": iamMessage.messageId,
                              @"isUnread": (iamMessage.unread? IAMTrue: IAMFalse),
                              @"isFavorite": (iamMessage.favorite? IAMTrue: IAMFalse)}];
    }
    
    (requestParams)[@"messages"]= messages;
    
    return requestParams;
}

-(id)parseResponse:(id)response forOperation:(AFHTTPRequestOperation *)operation
             error:(NSError *__autoreleasing *)error{

    // We expect a 204 status code on successful deletion.
    if (204 != operation.response.statusCode) {
        if (error) {
            NSString* description= [NSString stringWithFormat:
                                    @"Update failed for messages"];
            *error= [NSError errorWithDomain:@"ParsingDomain"
                                        code:101
                                    userInfo:@{NSLocalizedDescriptionKey: description}];
            
            return [NSNumber numberWithBool:NO];
        }
    }
    
    // the deletion was successful
    return [NSNumber numberWithBool:YES];
}

@end
