// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "MyCustomXCTestCase.h"
#import "IAMUtils.h"
#import "JSONParser.h"
#import "IAMMessage.h"
#import "IAMDelta.h"
#import "IAMMessageRequest.h"
#import "IAMDeltasUpdateRequest.h"
#import "OAuthToken.h"

@interface TestJSONParser : MyCustomXCTestCase

@end

@implementation TestJSONParser

- (void)setUp
{
    [super setUp];
}

- (void)tearDown
{
    // Put teardown code here; it will be run once, after the last test case.
    [super tearDown];
}

- (void)testParseMessagesList
{
    id json= [IAMUtils getJSONFromResource:@"messages-list"];
    
    NSArray* messages= [JSONParser getMessageListFromJSON:json];
    
    XCTAssert( 0 != [messages count], @"The message count is 0");
    
    NSLog(@"The messages are: %@", messages);
}

-(void)testParseMessageFromJSON{

    id json= [IAMUtils getJSONFromResource:@"message"];
    IAMMessage* message= [JSONParser getMessageFromJSON:(json)[@"message"]];
    
    XCTAssert(nil != message, @"ERROR: Could not parse message from JSON: %@", json);
    XCTAssert(nil != message.messageId, @"ERROR: Invalid message ID: %@", message.messageId);
    XCTAssert(nil != message.recipients && 0 < [message.recipients count],
              @"ERROR: Invalid message ID: %@", message.recipients);
    
    NSLog(@"The message is: %@", [message description]);

}

-(void)testParseDeltaUpdateFromJSON{

    id json= [IAMUtils getJSONFromResource:@"deltas"];
    
    NSDictionary* deltasJSON= (json)[@"deltaResponse"][@"delta"];

    NSArray* deltas = [JSONParser getDeltasFromJSON:deltasJSON];
    
    XCTAssert(nil != deltas, @"ERROR: Could not parse DeltaUpdate from JSON: %@", json);
    
    for (IAMDelta* delta in deltas) {
        
        for (IAMMessage* message in delta.deletions) {
            XCTAssert(nil != message.messageId, @"Invalid message ID: %@", message.messageId);
        }
        for (IAMMessage* message in delta.additions) {
            XCTAssert(nil != message.messageId, @"Invalid message ID: %@", message.messageId);
        }
        for (IAMMessage* message in delta.updates) {
            XCTAssert(nil != message.messageId, @"Invalid message ID: %@", message.messageId);
        }
    }
    
    NSLog(@"Deltas found are: %@", deltas);
}

-(void)testParseOAuthToken{
    // Token structure: http://developer.att.com/apis/oauth-2/docs/v1
    NSDictionary* tokenJSON= @{ @"access_token": @"some_token",
                                @"token_type": @"bearer",
                                @"expires_in": @"157679999",
                                @"refresh_token": @"refresh" };
    
    OAuthToken* token= [JSONParser getTokenFromJSON:tokenJSON];
    
    XCTAssertNotNil(token, @"Invalid Token");
    XCTAssertTrue([token.accessToken isEqualToString:@"some_token"],
                  @"Invalid Access Token: %@", token.accessToken);
    XCTAssertTrue([token.refreshToken isEqualToString:@"refresh"],
                  @"Invalid Refresh Token:%@",token.refreshToken);
    XCTAssertNotNil(token.expirationDate, @"Invalid Expiration Date");
    XCTAssertTrue([token.type isEqualToString:@"bearer"],
                  @"Invalid Type: %@", token.type);
    
    NSLog(@"Access Token: %@", [token description]);
}


@end
