// Copyright (c) 2013 AT&T (htpp://developer.att.com)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "TestNSObject+Description.h"
#import "IAMUtils.h"
#import "JSONParser.h"
#import "IAMDelta.h"
#import "NSObject+Description.h"
#import "MMSContent.h"

@implementation TestNSObject_Description{
    id deltasJSON;
    id messageJSON;
    NSArray* deltas;
    IAMMessage* message;
}

-(void)setUp{
    [super setUp];
    
    deltasJSON= [IAMUtils getJSONFromResource:@"deltas"];
    messageJSON= [IAMUtils getJSONFromResource:@"message"];

    deltas= [JSONParser getDeltasFromJSON:(deltasJSON)[@"deltaResponse"][@"delta"]];
    message= [JSONParser getMessageFromJSON:(messageJSON)[@"message"]];
}

-(void)testDescription{

    NSDictionary* dictionary= @{ @"string": @"string",
                     @"dictionary": @{@"string": @"string"},
                     @"array": @[@"one", @"two"]};
    
    NSLog(@"%@", [dictionary description]);
    NSLog(@"%@", dictionary);
    
    XCTAssert(nil != dictionary, @"Dictionary is nil.");
}

-(void)testMyDescriptionWithSimpleObject{
    
    XCTAssert(nil != message, @"Message is nil.");
    
    NSRange slashRange= [[message description] rangeOfString:@"\\"];
    XCTAssert(NSNotFound == slashRange.location, @"Found unescaped slashes");
    
    NSLog(@"%@", [message description]);
}

-(void)testMyDescriptionWithObject{

    XCTAssert(nil != deltas, @"No deltas.");
    
    NSLog(@"%@", [deltas getPropertiesDictionary]);

}

-(void)testGetPropertiesDictionary{
    
    NSArray* contents= message.mmsContent;
    
    // build the formatted string for the description
    NSDictionary* descriptionDic= [contents getPropertiesDictionary];

    XCTAssert( nil != descriptionDic, @"Description dictionary is nil.");
    NSRange slashRange= [[descriptionDic description] rangeOfString:@"\\"];
    XCTAssert( NSNotFound == slashRange.location, @"Invalid characters in description.");
    NSLog(@"%@", [descriptionDic description]);
    NSLog(@"%@", descriptionDic);
    
}
@end
