#!/bin/sh

# This script generates the distributable SDK Kit in two versions
# KIT_SRC: Constains all the source code + docs
# KIT_SRC: Constains the headers and the compiled binary + docs

# The path to the universal binary
ULIB=$1
# The path to the git root dir
GITROOT=$2
# The path to the SDK Kit
KIT_DIR=$GITROOT/"InAppMessagingKit"

# clean the Kit's directory
if [ -d $KIT_DIR ]; then
	echo "Removing old kits in $KIT_DIR"
	rm -Rf $KIT_DIR
fi

# create Kit's directory
mkdir $KIT_DIR

KIT_SRC=${KIT_DIR}/InAppMessagingKit-src
KIT_BIN=${KIT_DIR}/InAppMessagingKit-bin

mkdir $KIT_SRC
mkdir $KIT_BIN

###############################
#### CREATING THE KIT_SRC #####
###############################

# copy all sources to the Kit's directory
echo "Moving into $GITROOT..."
cd $GITROOT
echo "git checkout-index --prefix=${KIT_SRC}/ -f -a"
git checkout-index --prefix=${KIT_SRC}/ -f -a 

# install dependencies
# AFNetworking
cd $KIT_SRC/InAppMessaging
pod install
cd $GITROOT

# Base64
echo "Copying Base64 ..."
cp -Rf $GITROOT/InAppMessaging/Base64/* $KIT_SRC/InAppMessaging/Base64

###############################
#### CREATING THE KIT_BIN #####
###############################
# copy the Universal Library into the Kit's dir
echo "Copying the universal binary from $ULIB to $KIT_BIN/"
cp -Rf $ULIB $KIT_BIN
cp $GITROOT/README-bin.txt $KIT_BIN/README.txt


###############################
#### CREATING THE DOCS ########
###############################

DOCS_DIR=$GITROOT/InAppMessaging/docs/appledoc

# generate the docs
sh $GITROOT/gen-appledocs.sh $GITROOT/InAppMessaging/InAppMessaging $DOCS_DIR

# copy HTML & docsets to the Kit's dir
mkdir $KIT_SRC/doc
echo "Copying $DOCS_DIR to $KIT_SRC/doc"
cp -Rf $DOCS_DIR/html $DOCS_DIR/docset $KIT_SRC/doc
mkdir $KIT_BIN/doc
echo "Copying $DOCS_DIR to $KIT_BIN/doc"
cp -Rf $DOCS_DIR/html $DOCS_DIR/docset $KIT_BIN/doc

